html, body {
    margin: 0;
    padding: 0;
    font-family: sans-serif;
    color: #777788;
}

.clearfix {
    clear: both;
}

header {
    padding: 50px 20px;
    text-align: center;
}

header > .title {
    display: inline-block;
    color: #3771A1;
    font-size: 40px;
    font-weight: bold;
    text-shadow: 2px 2px 3px rgba(0, 0, 0, 0.2);
    vertical-align: middle;
}

header img {
    vertical-align: middle;
    margin-right: 24px;
}

main {
    margin: 0;
    padding: 15px;
}

footer {
    margin-top: 40px;
    border-top: 1px solid #CCCCCC;
    text-align: right;
}

footer a {
    display: inline-block;
    color: #777788;
    margin: 15px 10% 0 0;
}
